////////////////////////////////////////////////////////////////////////////////////////////////////
/// @file	JTAG_TAP.c
/// @author	Simon Qian(webmaster@SimonQian.com)
/// @date	Last Update on 2008-04-07
/// @brief	JTAG Test Access Port
/// @par	license
/// 		AVRminiProg is distributed under the terms and conditions of GPLv2, see License.txt
////////////////////////////////////////////////////////////////////////////////////////////////////
#include <avr/io.h>

#include "../../type.h"
#include "../../app_cfg.h"

#include "../../MCU/gpio.h"

#include "../../Delay/delay.h"

#include "../ISP/SPI/spi.h"
#include "JTAG_TAP.h"

#if AVRP_JTAG_SHOULD_ENABLE

#define JTAG_TAP_TCKP_DLY()			
#define JTAG_TAP_TCK_DLY()			

/// JTAG Clock
uint8 JTAG_Clock = 0;

/// JTAG TAP Change Status(by change TMS)
/// @param[in]	tms		data to be outputed on TMS
/// @param[in]	len		data length
void JTAG_TAP_ChangeStatus(uint8 tms,uint8 len)
{
	uint8 i,msk;

	for(i = 0;i < len;i++)
	{
		msk = (1 << i);
		if(tms & msk)
			JTAG_TAP_TMS_SET();
		else
			JTAG_TAP_TMS_CLR();

		JTAG_TAP_TCK_SET();
		JTAG_TAP_TCKP_DLY();
		JTAG_TAP_TCK_DLY();
		JTAG_TAP_TCK_CLR();
	}
}

/// JTAG TAP Shift Data below 1 byte
/// @param[in]	tdi		data to be outputed on TDI
/// @param[in]	len		data length
/// @return		data read on TDO
uint8 JTAG_TAP_ShiftDataInOut_1Byte(uint8 tdi,uint8 len)
{
	uint8 msk,tdo = 0;

	msk = 1;
	JTAG_TAP_TMS_CLR();
	while(len)
	{
		if(tdi & msk)
			JTAG_TAP_TDI_SET();
		else
			JTAG_TAP_TDI_CLR();

		if(JTAG_TAP_TDO_GET())
			tdo |= msk;

		if(len == 1)
			JTAG_TAP_TMS_SET();

		JTAG_TAP_TCK_SET();
		JTAG_TAP_TCKP_DLY();
		JTAG_TAP_TCK_DLY();
		JTAG_TAP_TCK_CLR();

		msk <<= 1;
		len--;
	}

	// Enter Update-IR or Update_DR Mode
//	JTAG_TAP_TMS_SET();
	JTAG_TAP_TCK_SET();
	JTAG_TAP_TCKP_DLY();
	JTAG_TAP_TCK_CLR();

	return tdo;
}

/// JTAG TAP Shift Data
/// @param[in]	tdi		data to be outputed on TDI
/// @param[out]	tdo		data read on TDO
/// @param[in]	len		data length
void JTAG_TAP_ShiftDataInOut(uint8 *tdi,uint8 *tdo,uint16 len)
{
	uint8 i = 0,tmp,len_remain = len % 8,tdi_tmp;
	uint16 bytes = len / 8;

	JTAG_TAP_TMS_CLR();

	if(!len_remain)
	{
		len_remain = 8;
		bytes--;
	}

	if(bytes)
	{
		SPI_Init();
#if 1
		SPCR = (1 << SPE) | (1 << MSTR) | (1 << DORD);
		SPSR = (1 << SPI2X);
#else
		SPI_SetClk(0);
		SPCR |= (1 << DORD);
#endif

		if(tdi)
			SPI_SetData(tdi[i]);
		else
			SPI_SetData(0);
		bytes--;
		while(bytes)
		{
			SPI_WaitReady();
			if(tdo)
				tdo[i] = SPI_GetData();
			i++;
			if(tdi)
				SPI_SetData(tdi[i]);
			else
				SPI_SetData(0);

			bytes--;
		}
		SPI_WaitReady();
		if(tdo)
			tdo[i] = SPI_GetData();
		i++;
		SPI_Disable();
	}

	if(tdi)
		tdi_tmp = tdi[i];
	else
		tdi_tmp = 0;
	tmp = JTAG_TAP_ShiftDataInOut_1Byte(tdi_tmp,len_remain);
	if(tdo)
		tdo[i] = tmp;
}

/// JTAG TAP Send Instruction
/// @param[in]	instr	Instruction to send
/// @param[in]	len		Length of Instruction
void JTAG_TAP_Instr(uint8 instr,uint8 len)
{
	JTAG_TAP_Enter_IR_Mode();

	JTAG_TAP_ShiftDataInOut_1Byte(instr,len);
}

/// JTAG TAP Transive Data
/// @param[in]	tdi		data to be sent
/// @param[out]	tdo		data to be received
/// @param[in]	len		len of data
/// @param[in]	idle	whether to enter idle mode after operation
void JTAG_TAP_Data(uint8 *tdi,uint8 *tdo,uint16 len,uint8 idle)
{
	JTAG_TAP_Enter_DR_Mode();

	JTAG_TAP_ShiftDataInOut(tdi,tdo,len);

	if(idle)
		JTAG_TAP_Enter_Idle_Mode();
}

/// JTAG TAP Transive Data below 1 byte
/// @param[in]	tdi		data to be sent
/// @param[in]	len		len of data
/// @param[in]	idle	whether to enter idle mode after operation
uint8 JTAG_TAP_Data_1Byte(uint8 tdi,uint8 len,uint8 idle)
{
	uint8 tdo;

	JTAG_TAP_Enter_DR_Mode();
	tdo = JTAG_TAP_ShiftDataInOut_1Byte(tdi,len);

	if(idle)
		JTAG_TAP_Enter_Idle_Mode();

	return tdo;
}

/// JTAG TAP Transive Data below 2 bytes
/// @param[in]	tdi		data to be sent
/// @param[in]	len		len of data
/// @param[in]	idle	whether to enter idle mode after operation
uint16 JTAG_TAP_Data_2Bytes(uint16 tdi,uint8 len,uint8 idle)
{
	uint16 tdo;

	JTAG_TAP_Enter_DR_Mode();

	// Data Out
	JTAG_TAP_ShiftDataInOut((uint8*)&tdi,(uint8*)&tdo,len);

	if(idle)
		JTAG_TAP_Enter_Idle_Mode();

	return tdo;
}

/// JTAG TAP Initialization
/// @return
void JTAG_TAP_Init(void)
{
/*
	JTAG_TAP_TMS_SETGPIO();
	JTAG_TAP_TCK_SETGPIO();
	JTAG_TAP_TDI_SETGPIO();
	JTAG_TAP_TDO_SETGPIO();
	JTAG_TAP_TRST_SETGPIO();
*/
	JTAG_TAP_TMS_SET();
	JTAG_TAP_TMS_SETOUTPUT();

	JTAG_TAP_TCK_CLR();
	JTAG_TAP_TCK_SETOUTPUT();

	JTAG_TAP_TDI_SET();
	JTAG_TAP_TDI_SETOUTPUT();

	JTAG_TAP_TDO_SETINPUT();
/*
	JTAG_TAP_TRST_SET();
	JTAG_TAP_TRST_SETOUTPUT();
*/
}

/// JTAG TAP Finilization
/// @return
void JTAG_TAP_Fini(void)
{
	JTAG_TAP_TMS_SETINPUT();
	JTAG_TAP_TCK_SETINPUT();
	JTAG_TAP_TDI_SETINPUT();
	JTAG_TAP_TDO_SETINPUT();
//	JTAG_TAP_TRST_SETINPUT();
}
#endif		// #if AVRP_JTAG_SHOULD_ENABLE
