////////////////////////////////////////////////////////////////////////////////////////////////////
/// @file	app_cfg_USBasp.h
/// @author	Simon Qian(webmaster@SimonQian.com)
/// @date	Last Update on 2008-04-07
/// @brief	application configuration file for AVRminiProg_USBasp
/// @par	license
/// 		AVRminiProg is distributed under the terms and conditions of GPLv2, see License.txt
////////////////////////////////////////////////////////////////////////////////////////////////////

/******************************** AVRminiProgUSBasp ********************************/
//  AVRminiProgUSBasp is compatible with USBasp in hardware. For AVRminiProg need more
//code, so Mega48 can NOT be used. If JTAG programming is needed, use Mega168.

#ifndef APP_CFG_INCLUDED

#define APP_CFG_INCLUDED

#define AVR_DEVICE					(AVR_DEVICE_AVRISP_MKII)//(AVR_DEVICE_AVRISP_MKII | AVR_DEVICE_AVRPHENIX | AVR_DEVICE_AVRPHENIX_ONLY)

// if AVR_DEVICE_AVRISP_MKII is sellected, only ISP could be enabled, others will be ignored
// if AVR_DEVICE_JTAGICE_MKII is sellected, HV Programming will be ignored
#define AVR_PROG_ISP_EN				1
#define AVR_PROG_JTAG_EN			0
#define AVR_PROG_HVPP_EN			0			// NOT supported by USBasp
#define AVR_PROG_HVSP_EN			0			// NOT supported by USBasp
#define AVRX_PROG_JTAG_EN			0
#define AVRX_PROG_PDI_EN			0
#define AVR32_PROG_JTAG_EN			0

#define AVR_DBG_JTAG_EN				0			// NOT implemented, don't use
#define AVR_DBG_DW_EN				0			// NOT implemented, don't use

/*************************** ADC *****************************/
/// devices like USBasp has no AD connected to target VCC,just define AD_En to 0
#define AD_En						1

/// VREF can be ADC_VREF_AVCC or ADC_VREF_BANDGAP
#define AD_VCC_MUX					(ADC_VREF_BANDGAP | ADC_CHANNEL_ADC3)
/// Voltage of AD Reference(in 100mA,so 11 means 1.1V)
/// If Internal Bandgap is used, Mega88 is 11, while Mega8 is 25
#define ADVref						25
/// target VCC ------ R1 ------ ADC
///                           |
///        GND ------ R2 ------
/// 
/// AD_VCC_RATE equals to (resistor1 + resistor2) / resistor2
/// for Reference Voltage (AVCC will change according to different current load, BANDGAP will not):
/// if AVCC is used, resistor1 = resistor2 = 10K					AD_VCC_RATE should be 2
/// if BANDGAP is used, resistor1 = 10K, resistor2 = 2K			AD_VCC_RATE should be 6
/// I use AVCC as reference, and resistor1 = 10K, resistor2 = 2K	AD_VCC_RATE is 6
/// Of cource other values are also acceptable, just use different R1 and R2
#define AD_VCC_RATE					6

/*************************** LED *****************************/
// LED used to indicate connected to PC
#define LED_Green					P(1)
// LED used to indicate Target VCC
#define LED_Red						P(0)

/// LED Initialization
#define LED_Init()					do{PortC_SetPin(LED_Green | LED_Red);PortC_SetOutput(LED_Green | LED_Red);}while(0)
/// Turn LED on
#define LED_On(led)					PortC_ClrPin(led)
/// Turn LED off
#define LED_Off(led)				PortC_SetPin(led)
/// LED Is on?
#define LED_IsOn(led)				!PortC_GetPin(led)


/*************************** JTAG *****************************/
/// STRONGLY recommended to be the same pin as SPI_SS
#define JTAG_TAP_TMS				P(2)
// MUST be SPI0 as ISP interface,so that one adaptor is enough for both ISP and JTAG.
/// MUST be the same PIN as SPI_SCK
#define JTAG_TAP_TCK				P(5)
/// MUST be the same PIN as SPI_MOSI
#define JTAG_TAP_TDI				P(3)
/// MUST be the same PIN as SPI_MISO
#define JTAG_TAP_TDO				P(4)

/// Set TMS Pin to Output
#define JTAG_TAP_TMS_SETOUTPUT()	PortB_SetOutput(JTAG_TAP_TMS)
/// Set TMS Pin to Input
#define JTAG_TAP_TMS_SETINPUT()		PortB_SetInput(JTAG_TAP_TMS)
/// Set TMS Pin High
#define JTAG_TAP_TMS_SET()			PortB_SetPin(JTAG_TAP_TMS)
/// Set TMS Pin Low
#define JTAG_TAP_TMS_CLR()			PortB_ClrPin(JTAG_TAP_TMS)

/// Set TCK Pin to Output
#define JTAG_TAP_TCK_SETOUTPUT()	PortB_SetOutput(JTAG_TAP_TCK)
/// Set TCK Pin to Input
#define JTAG_TAP_TCK_SETINPUT()		PortB_SetInput(JTAG_TAP_TCK)
/// Set TCK Pin High
#define JTAG_TAP_TCK_SET()			PortB_SetPin(JTAG_TAP_TCK)
/// Set TCK Pin low
#define JTAG_TAP_TCK_CLR()			PortB_ClrPin(JTAG_TAP_TCK)

/// Set TDI Pin to Output
#define JTAG_TAP_TDI_SETOUTPUT()	PortB_SetOutput(JTAG_TAP_TDI)
/// Set TDI Pin to Input
#define JTAG_TAP_TDI_SETINPUT()		PortB_SetInput(JTAG_TAP_TDI)
/// Set TDI Pin High
#define JTAG_TAP_TDI_SET()			PortB_SetPin(JTAG_TAP_TDI)
/// Set TDI Pin Low
#define JTAG_TAP_TDI_CLR()			PortB_ClrPin(JTAG_TAP_TDI)

/// Set TDO Pin to Input
#define JTAG_TAP_TDO_SETINPUT()		PortB_SetInput(JTAG_TAP_TDO)
/// Get TDO Pin
#define JTAG_TAP_TDO_GET()			PortB_GetPin(JTAG_TAP_TDO)
/// Pullup TDO Pin
#define JTAG_TAP_TDO_SET()			PortB_SetPin(JTAG_TAP_TDO)
/// Cancel pullup on TDO Pin
#define JTAG_TAP_TDO_CLR()			PortB_ClrPin(JTAG_TAP_TDO)

/*************************** ISP *****************************/
/// if error occurs during programming with AutoSpeed set,
/// just set ISP speed in AVRSutdio and AVRminiProg will disable AutoSpeed
/// If U don't like it, change ISP_AutoSpeed_En to 0
/// If used, an 12M or 15M osc is recommended
/// There is some other configurations in ISP.h file. If problems occur, define this to 0
#define ISP_AutoSpeed_En			1

/// SPI Port for ISP(SPI0 Must be used)
#define SPI_MISO					P(4)
/// SPI Port for ISP(SPI0 Must be used)
#define SPI_MOSI					P(3)
/// SPI Port for ISP(SPI0 Must be used)
#define SPI_SCK						P(5)
/// SPI Port for ISP(SPI0 Must be used)
#define SPI_SS						P(2)

/// in Master SPI mode,if SS is input, SS MUST be held high to ensure master SPI operation
/// so configure SS to output or input with pullup, just sellect one below
/// undefine both if SS pin is used as RESET control like USBasp
/// SPI_SS is strongly recommended as ISP_RST and JTAG_TAP_TMS, and undefine both
#define SPI_SS_Init()				//do{PortB_SetPin(SPI_SS);PortB_SetInput(SPI_SS);}while(0)
//#define SPI_SS_Init()				PortB_SetOutput(SPI_SS)

/// Set MOSI Pin to Output
#define SPI_SetMOSIOutput()			PortB_SetOutput(SPI_MOSI)
/// Set MOSI Pin to Input
#define SPI_SetMOSIInput()			PortB_SetInput(SPI_MOSI)
/// Set MOSI Pin High
#define SPI_SetMOSI()				PortB_SetPin(SPI_MOSI)
/// Set MOSI Pin Low
#define SPI_ClrMOSI()				PortB_ClrPin(SPI_MOSI)

/// Set MISO Pin Input
#define SPI_SetMISOInput()			PortB_SetInput(SPI_MISO)
/// Get MISO Pin
#define SPI_GetMISO()				PortB_GetPin(SPI_MISO)

/// Set SCK Pin to Output
#define SPI_SetSCKOutput()			PortB_SetOutput(SPI_SCK)
/// Set SCK Pin to Input
#define SPI_SetSCKInput()			PortB_SetInput(SPI_SCK)
/// Set SCK Pin High
#define SPI_SetSCK()				PortB_SetPin(SPI_SCK)
/// Set SCK Pin Low
#define SPI_ClrSCK()				PortB_ClrPin(SPI_SCK)

/// Reset Pin
#define ISP_RST						P(2)
/// Set Reset Pin High
#define ISP_SetRST()				PortB_SetPin(ISP_RST)
/// Set Reset Pin Low
#define ISP_ClrRST()				PortB_ClrPin(ISP_RST)
/// Set Reset Pin to Output
#define ISP_SetRSTOutput()			PortB_SetOutput(ISP_RST)
/// Set Reset Pin to Input
#define ISP_SetRSTInput()			PortB_SetInput(ISP_RST)

/*************************** VHPP *****************************/

/*************************** HVSP *****************************/


/*************************** AVRUSB *****************************/
/* ---------------------------- Hardware Config ---------------------------- */

#define USB_CFG_IOPORTNAME      B
/* This is the port where the USB bus is connected. When you configure it to
 * "B", the registers PORTB, PINB and DDRB will be used.
 */
#define USB_CFG_DMINUS_BIT      0
/* This is the bit number in USB_CFG_IOPORT where the USB D- line is connected.
 * This may be any bit in the port.
 */
#define USB_CFG_DPLUS_BIT       1
/* This is the bit number in USB_CFG_IOPORT where the USB D+ line is connected.
 * This may be any bit in the port. Please note that D+ must also be connected
 * to interrupt pin INT0!
 */
#define USB_CFG_CLOCK_KHZ       (F_CPU/1000)
/* Clock rate of the AVR in MHz. Legal values are 12000, 15000, 16000 or 16500.
 * The 16.5 MHz version of the code requires no crystal, it tolerates +/- 1%
 * deviation from the nominal frequency. All other rates require a precision
 * of 2000 ppm and thus a crystal!
 * Default if not specified: 12 MHz
 */
/* ----------------------- Optional Hardware Config ------------------------ */

/* #define USB_CFG_PULLUP_IOPORTNAME   D */
/* If you connect the 1.5k pullup resistor from D- to a port pin instead of
 * V+, you can connect and disconnect the device from firmware by calling
 * the macros usbDeviceConnect() and usbDeviceDisconnect() (see usbdrv.h).
 * This constant defines the port on which the pullup resistor is connected.
 */
/* #define USB_CFG_PULLUP_BIT          4 */
/* This constant defines the bit number in USB_CFG_PULLUP_IOPORT (defined
 * above) where the 1.5k pullup resistor is connected. See description
 * above for details.
 */

/*************************** KEY *****************************/
//#define PROGKEY_Pin					(1 << 0)
/// Program Key Initialization
#define PROGKEY_Init()				do{PortC_SetPin(P(4));PortC_ClrPin(P(5));PortC_SetOutput(P(5));PortC_SetInput(P(4));}while(0)
/// Check Program Key
#define PROGKEY_IsDown()			!PortC_GetPin(P(4))

#endif		// #ifndef APP_CFG_INCLUDED
